@@ -24,6 +24,7 @@
 #include <glib/gstdio.h> //For g_remove().
 #include <glom/libglom/spawn_with_feedback.h>
 #include <glom/libglom/utils.h>
+//#include <glom/libglom/avahi_publisher.h>
 #include <glibmm/i18n.h>
 
 #include <sys/types.h>
@@ -31,15 +32,8 @@
 #include <sys/socket.h>
 #include <netinet/in.h> //For sockaddr_in
 
-#include <avahi-client/client.h>
-#include <avahi-client/publish.h>
-#include <avahi-common/error.h>
-#include <avahi-common/timeval.h>
-#include <avahi-common/alternative.h>
-#include <avahi-glib/glib-watch.h>
-#include <avahi-glib/glib-malloc.h>
 
-#include "gst-package.h"
+#include <glom/libglom/gst-package.h>
 
 #include "config.h"
 
@@ -53,7 +47,7 @@ namespace Glom
 # local is for Unix domain socket connections only\n\
 local   all         all                               ident sameuser\n\
 # IPv4 local connections:\n\
-host    all         all         127.0.0.1/32          md5\n\
+host    all         all         0.0.0.0/0          md5\n\
 # IPv6 local connections:\n\
 host    all         all         ::1/128               md5\n"
 
@@ -132,10 +126,7 @@ ConnectionPool::ConnectionPool()
 : m_self_hosting_active(false),
   m_sharedconnection_refcount(0),
   m_ready_to_connect(false),
-  m_pFieldTypes(0),
-  m_avahi_group(0),
-  m_avahi_client(0),
-  m_avahi_mainloop(0)
+  m_pFieldTypes(0)
 {
   m_list_ports.push_back("5432"); //Ubuntu Breezy seems to default to this for Postgres 7.4, and this is probably the default for most postgres installations, including Fedora.
 
@@ -512,7 +503,16 @@ float ConnectionPool::get_postgres_server_version()
   return m_postgres_server_version;
 }
 
-bool ConnectionPool::directory_exists(const std::string& uri)
+/*
+bool ConnectionPool::directory_exists_filepath(const std::string& filepath)
+{
+  const std::string uri = Glib::filename_to_uri(filepath);
+
+  return directory_exists_uri(uri);
+}
+*/
+
+bool ConnectionPool::directory_exists_uri(const std::string& uri)
 {
   Glib::RefPtr<Gnome::Vfs::Uri> vfsuri = Gnome::Vfs::Uri::create(uri);
   return vfsuri->uri_exists();
@@ -525,7 +525,7 @@ bool ConnectionPool::start_self_hosting()
 
   const std::string dbdir_uri = m_self_hosting_data_uri;
 
-  if(!(directory_exists(dbdir_uri)))
+  if(!(directory_exists_uri(dbdir_uri)))
   {
     std::cerr << "ConnectionPool::create_self_hosting(): The directory could not be found: " << dbdir_uri << std::endl;
     return false;
@@ -536,7 +536,7 @@ bool ConnectionPool::start_self_hosting()
 
   const std::string dbdir_data = Glib::build_filename(dbdir, "data");
   const std::string dbdir_data_uri = Glib::filename_to_uri(dbdir_data);
-  if(!(directory_exists(dbdir_data_uri)))
+  if(!(directory_exists_uri(dbdir_data_uri)))
   {
     std::cerr << "ConnectionPool::create_self_hosting(): The data sub-directory could not be found." << dbdir_data_uri << std::endl;
     return false;
@@ -625,7 +625,7 @@ void ConnectionPool::stop_self_hosting()
 }
 
 
-bool ConnectionPool::create_self_hosting()
+bool ConnectionPool::create_self_hosting(Gtk::Window* parent_window)
 {
   if(m_self_hosting_data_uri.empty())
   {
@@ -636,6 +636,17 @@ bool ConnectionPool::create_self_hosting()
   //Get the filepath of the directory that we should create:
   const std::string dbdir_uri = m_self_hosting_data_uri;
   //std::cout << "debug: dbdir_uri=" << dbdir_uri << std::endl;
+
+  if(directory_exists_uri(dbdir_uri))
+  { 
+    if(parent_window)
+    {
+      Utils::show_ok_dialog(_("Directory Already Exists"), _("There is an existing directory with the same name as the directory that should be created for the new database files. You should specify a different filename to use a new directory instead."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+    
+    return false;
+  }
+
   const std::string dbdir = Glib::filename_from_uri(dbdir_uri);
   //std::cout << "debug: dbdir=" << dbdir << std::endl;
   g_assert(!dbdir.empty());
@@ -647,6 +658,12 @@ bool ConnectionPool::create_self_hosting()
   {
     std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir << std::endl;
     perror("Error from g_mkdir_with_parents");
+
+    if(parent_window)
+    {
+       Utils::show_ok_dialog(_("Could Not Create Directory"), _("There was an error when attempting to create the directory for the new database files."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+
     return false;
   }
 
@@ -655,6 +672,11 @@ bool ConnectionPool::create_self_hosting()
   mkdir_succeeded = g_mkdir_with_parents(dbdir_config.c_str(), 0770);
   if(mkdir_succeeded == -1)
   {
+    if(parent_window)
+    {
+       Utils::show_ok_dialog(_("Could Not Create Configuration Directory"), _("There was an error when attempting to create the configuration directory for the new database files."), *parent_window, Gtk::MESSAGE_ERROR);
+    }
+
     std::cerr << "Error from g_mkdir_with_parents() while trying to create directory: " << dbdir_config << std::endl;
     perror("Error from g_mkdir_with_parents");
     return false;
@@ -867,266 +889,34 @@ bool ConnectionPool::install_postgres(Gtk::Window* parent_window)
 }
 
 
-
-static void entry_group_callback(AvahiEntryGroup *g, AvahiEntryGroupState state, void *userdata)
-{
-    ConnectionPool* self = (ConnectionPool*)userdata;
-
-    assert(g == self->m_avahi_group || self->m_avahi_group == NULL);
-
-    /* Called whenever the entry group state changes */
-
-    switch (state) {
-        case AVAHI_ENTRY_GROUP_ESTABLISHED :
-            /* The entry group has been established successfully */
-            fprintf(stderr, "Service '%s' successfully established.\n", self->m_avahi_service_name.c_str());
-            break;
-
-        case AVAHI_ENTRY_GROUP_COLLISION : {
-   
-            /* A service name collision happened. Let's pick a new name */
-            char* n = avahi_alternative_service_name(self->m_avahi_service_name.c_str());
-            self->m_avahi_service_name = n;
-            avahi_free(n);
-            n = 0;
-            
-            fprintf(stderr, "Service name collision, renaming service to '%s'\n", self->m_avahi_service_name.c_str());
-            
-            /* And recreate the services */
-            self->avahi_create_services(avahi_entry_group_get_client(g));
-            break;
-        }
-
-        case AVAHI_ENTRY_GROUP_FAILURE :
-
-            fprintf(stderr, "Entry group failure: %s\n", avahi_strerror(avahi_client_errno(avahi_entry_group_get_client(g))));
-
-            /* Some kind of failure happened while we were registering our services */
-            g_main_loop_quit (self->m_avahi_mainloop);
-            break;
-
-        case AVAHI_ENTRY_GROUP_UNCOMMITED:
-        case AVAHI_ENTRY_GROUP_REGISTERING:
-            ;
-    }
-}
-
-void ConnectionPool::avahi_create_services(AvahiClient *c)
-{
-    int ret;
-    assert(c);
-
-    bool failed = false;
-
-    /* If this is the first time we're called, let's create a new entry group */
-    if (!m_avahi_group)
-        if (!(m_avahi_group = avahi_entry_group_new(c, entry_group_callback, this /* user_data */))) {
-            fprintf(stderr, "avahi_entry_group_new() failed: %s\n", avahi_strerror(avahi_client_errno(c)));
-            failed = true;
-        }
-    
-    if(!failed)
-    {
-      fprintf(stderr, "Adding service '%s'\n", m_avahi_service_name.c_str());
-    }
-
-    const int port = atoi(m_port.c_str());
-
-    /* Add the service for Glom: */
-    if (!failed && (ret = avahi_entry_group_add_service(m_avahi_group, AVAHI_IF_UNSPEC, AVAHI_PROTO_UNSPEC, (AvahiPublishFlags)0, m_avahi_service_name.c_str(), "_glom._tcp", NULL, NULL, port, NULL)) < 0) {
-        fprintf(stderr, "Failed to add _ipp._tcp service: %s\n", avahi_strerror(ret));
-        failed = true;
-    }
-    
-    /* Tell the server to register the service */
-    if (!failed && (ret = avahi_entry_group_commit(m_avahi_group)) < 0) {
-        fprintf(stderr, "Failed to commit entry_group: %s\n", avahi_strerror(ret));
-        failed = true;
-    }
-
-    if(failed)
-    {
-      g_main_loop_quit (m_avahi_mainloop);
-    }
-}
-
-
-/* Callback for Avahi API Timeout Event */
-static void
-avahi_timeout_event (AVAHI_GCC_UNUSED AvahiTimeout *timeout, AVAHI_GCC_UNUSED void *userdata)
-{
-    g_message ("Avahi API Timeout reached!");
-}
-
-/* Callback for GLIB API Timeout Event */
-static gboolean
-avahi_timeout_event_glib (void *userdata)
-{
-    GMainLoop *loop = (GMainLoop*)userdata;
-
-    g_message ("GLIB API Timeout reached, quitting main loop!");
-    
-    /* Quit the application */
-    g_main_loop_quit (loop);
-
-    return FALSE; /* Don't re-schedule timeout event */
-}
-
-/* Callback for state changes on the Client */
-static void
-avahi_client_callback (AvahiClient *client, AvahiClientState state, void *userdata)
-{
-  ConnectionPool* self = (ConnectionPool*)userdata;
-
-  g_message ("Avahi Client State Change: %d", state);
-
-   /* Called whenever the client or server state changes */
-   switch (state) {
-        case AVAHI_CLIENT_S_RUNNING:
-        
-            /* The server has startup successfully and registered its host
-             * name on the network, so it's time to create our services */
-            if (!self->m_avahi_group)
-                self->avahi_create_services(client);
-            break;
-
-        case AVAHI_CLIENT_FAILURE:
-            
-            fprintf(stderr, "Client failure: %s\n", avahi_strerror(avahi_client_errno(client)));
-            g_main_loop_quit (self->m_avahi_mainloop);
-            
-            break;
-
-        case AVAHI_CLIENT_S_COLLISION:
-        
-            /* Let's drop our registered services. When the server is back
-             * in AVAHI_SERVER_RUNNING state we will register them
-             * again with the new host name. */
-            
-        case AVAHI_CLIENT_S_REGISTERING:
-
-            /* The server records are now being established. This
-             * might be caused by a host name change. We need to wait
-             * for our own records to register until the host name is
-             * properly esatblished. */
-            
-            if (self->m_avahi_group)
-                avahi_entry_group_reset(self->m_avahi_group);
-            
-            break;
-
-        case AVAHI_CLIENT_CONNECTING:
-            ;
-    }
-}
-
 /** Advertise self-hosting via avahi:
  */
 void ConnectionPool::avahi_start_publishing()
 {
-  m_avahi_service_name = "glom_selfhosted_" + get_database();
+  //This is commented-out for now, until we make it stable.
 
-  /* Optional: Tell avahi to use g_malloc and g_free */
-  avahi_set_allocator (avahi_glib_allocator ());
+  /*
+  if(m_avahi_publisher)
+    return;
 
-  /* Create the GLIB main loop */
-  m_avahi_mainloop = g_main_loop_new (NULL, FALSE);
+  const Glib::ustring avahi_service_name = "glom_selfhosted_" + get_database();
+  const int port = atoi(m_port.c_str());
 
-  /* Create the GLIB Adaptor */
-  AvahiGLibPoll* glib_poll = avahi_glib_poll_new (NULL, G_PRIORITY_DEFAULT);
-  const AvahiPoll* poll_api = avahi_glib_poll_get (glib_poll);
-
-  /* Example, schedule a timeout event with the Avahi API */
-  struct timeval tv;
-  avahi_elapse_time (&tv,                         /* timeval structure */
-            1000,                                   /* 1 second */
-            0);                                     /* "jitter" - Random additional delay from 0 to this value */
-
-  poll_api->timeout_new (poll_api,                /* The AvahiPoll object */
-                      &tv,                          /* struct timeval indicating when to go activate */
-                      avahi_timeout_event,          /* Pointer to function to call */
-                      NULL);                        /* User data to pass to function */
-
-  /* Schedule a timeout event with the glib api */
-  g_timeout_add (5000,                            /* 5 seconds */
-            avahi_timeout_event_glib,               /* Pointer to function callback */
-            m_avahi_mainloop);                                  /* User data to pass to function */
-
-  /* Create a new AvahiClient instance */
-  int error = 0;
-  m_avahi_client = avahi_client_new (poll_api,            /* AvahiPoll object from above */
-                               (AvahiClientFlags)0,
-            avahi_client_callback,                  /* Callback function for Client state changes */
-            this,                                   /* User data */
-            &error);                                /* Error return */
-
-  bool failed = false;
-  const char* version = 0;
-
-  /* Check the error return code */
-  if (m_avahi_client == NULL)
-  {
-    /* Print out the error string */
-    g_warning ("Error initializing Avahi: %s", avahi_strerror (error));
-
-    failed = true;
-  }
-  else
-  {
-    /* Make a call to get the version string from the daemon */
-    const char* version = avahi_client_get_version_string (m_avahi_client);
-
-    /* Check if the call suceeded */
-    if (version == NULL)
-    {
-      g_warning ("Error getting version string: %s", avahi_strerror (avahi_client_errno (m_avahi_client)));
-
-      failed = true;
-    }
-  }
-    
-  if(!failed)
-  {
-    if(version)
-      g_message ("Avahi Server Version: %s", version);
-
-    /* Start the GLIB Main Loop */
-    g_main_loop_run (m_avahi_mainloop);
-  }
-  else
-  {
-    /* Clean up */
-    if(m_avahi_mainloop)
-    {
-      g_main_loop_unref (m_avahi_mainloop);
-      m_avahi_mainloop = 0;
-    }
-
-    if(m_avahi_client)
-    {
-      avahi_client_free (m_avahi_client);
-      m_avahi_client = 0;
-    }
-
-    if(glib_poll)
-      avahi_glib_poll_free (glib_poll);
-  }
+  m_avahi_publisher = new AvahiPublisher(avahi_service_name, "_glom._tcp", port);
+  */
 }
 
 void ConnectionPool::avahi_stop_publishing()
 {
-    if(m_avahi_client)
-    {
-      avahi_client_free (m_avahi_client);
-      m_avahi_client = 0;
-    }
+  //This is commented-out for now, until we make it stable.
 
-    /* Clean up */
-    if(m_avahi_mainloop)
-    {
-      g_main_loop_unref (m_avahi_mainloop);
-      m_avahi_mainloop = 0;
-    }
+  /*
+  if(m_avahi_publisher)
+  {
+    delete m_avahi_publisher;
+    m_avahi_publisher = 0;
+  }
+  */
 }
 
 
